
from glouton.modules.observationModuleBase import ObservationModuleBase

class TestModule(ObservationModuleBase):

    def runAfterDownload(self, file_name, full_path, observation):
        print('executed after ' +  file_name)


from glouton.infrastructure.satnogNetworkClient import SatnogNetworkClient
from glouton.commands.module.moduleCommandParams import ModuleCommandParams
from glouton.commands.module.moduleCommand import ModuleCommand
import os

class DownloadCommand:
    def __init__(self, params, modules_commands):
        self.full_path = os.path.join(params.working_dir, params.sub_folder)
        self.modules = params.modules
        self.modules_commands = modules_commands

    def download(self):
        raise NotImplementedError()

    def runModulesAfterDownload(self, file_name):
        raise NotImplementedError()




class DownloadCommandParams:
    def __init__(self, working_dir=None, sub_folder=None, modules=None):
        self.working_dir = working_dir
        self.sub_folder = sub_folder
        self.modules = modules



from glouton.modules.observationModuleBase import ObservationModuleBase
from time import strftime, strptime
from datetime import datetime

# This module writes out decoded telemetry frames in the same format
# provided by the telemetry download option on db.satnogs.org.
class CSV(ObservationModuleBase):

    def runAfterDownload(self, file_name, full_path, observation):
        csv_file = full_path + '/' + file_name + '.csv'

        # extract timestamp from binary frame file name
        # the filename consists of three parts:
        # data_843421_2019-07-20T13-21-51
        # 1. prefix "data"
        # 2. observation id
        # 3. timestamp of the recorded frame in UTC
        # after the split the array index [2] contains the timestamp
        dobj = datetime.strptime(file_name.split('_')[2], '%Y-%m-%dT%H-%M-%S')
        obs_time = dobj.strftime('%Y-%m-%d %H:%M:%S')

        with open(full_path + '/' + file_name, 'rb') as f:
            hexdump = f.read().hex().upper()
        with open(csv_file, 'w') as f:
            f.write(obs_time + '|' + hexdump + '\n')



from glouton.shared.logger import logger

class DownloadWorker:
    def __init__(self, queue, download_status):
        self._commands = queue
        self.__download_status = download_status

    def execute(self):
        self.__download_status.set()
        try:       
            while self._commands.empty() == False:
                command = self._commands.get()
                command.download()
                self._commands.task_done()

        except Exception as ex:
            logger.Error(ex)

        self.__download_status.clear()



from glouton.infrastructure.satnogDbClient import SatnogDbClient
from glouton.commands.download.downloadCommand import DownloadCommand
from glouton.commands.module.telemetryModuleCommandParams import TelemetryModuleCommandParams
from glouton.commands.module.telemetryModuleCommand import TelemetryModuleCommand
import os

class DownloadTelemetryCommand(DownloadCommand):
    def __init__(self, params, telemetry, modules_commands):
        DownloadCommand.__init__(self, params, modules_commands)
        self.telemetry = telemetry
        self.client = SatnogDbClient()

    def download(self):
        raise NotImplementedError()

    def runModulesAfterDownload(self, frame):
        if self.modules is not None:
            for module in self.modules:
                cmd_parameters = TelemetryModuleCommandParams(
                    frame, self.full_path, self.telemetry, module
                )
                moduleCommand = TelemetryModuleCommand(cmd_parameters)
                self.modules_commands.put(moduleCommand)



import pickle

class SessionService:
    def __init__(self):
        self.SESSION_FILENAME = ".session"

    def save_program_parameters(self, cmd):
        with open(self.SESSION_FILENAME, 'wb') as f:
            pickle.dump(cmd, f, pickle.DEFAULT_PROTOCOL)

    def load_last_program_parameters(self):
        with open(self.SESSION_FILENAME, "rb") as f:
            return pickle.load(f)



from glouton.infrastructure.satnogNetworkClient import SatnogNetworkClient
from glouton.commands.download.downloadCommand import DownloadCommand
from glouton.commands.module.observationModuleCommandParams import ObservationModuleCommandParams
from glouton.commands.module.observationModuleCommand import ObservationModuleCommand
import os

class DownloadObservationCommand(DownloadCommand):
    def __init__(self, params, observation, modules_commands):
        DownloadCommand.__init__(self, params, modules_commands)
        self.observation = observation
        self.client = SatnogNetworkClient()

    def download(self):
        raise NotImplementedError()

    def runModulesAfterDownload(self, file_name):
        if self.modules is not None:
            for module in self.modules:
                cmd_parameters = ObservationModuleCommandParams(
                    file_name, self.full_path, self.observation, module
                )
                moduleCommand = ObservationModuleCommand(cmd_parameters)
                self.modules_commands.put(moduleCommand)



import os

class ModuleBase:
    def __init__(self, working_dir):
        self.working_dir = working_dir

    def runAfterDownload(self, file_name, full_path, observation):
        raise NotImplementedError()



from glouton.commands.download.downloadTelemetryCommand import DownloadTelemetryCommand
from glouton.shared import fileHelper
from glouton.shared.logger import logger
import os
import ntpath

class FrameDownloadCommand(DownloadTelemetryCommand):
    def __init__(self, params, telemetry, modules_commands):
        DownloadTelemetryCommand.__init__(self, params, telemetry, modules_commands)
        self.__json_id = "frame"

    def download(self):
        frame = self.telemetry[self.__json_id]
        if not frame:
            logger.Info('no frame found for the telemetry of ' + self.telemetry['timestamp'])
            return

        fileHelper.create_dir_if_not_exist(self.full_path)
        
        self.runModulesAfterDownload(frame)


from queue import Queue
from glouton.infrastructure.satnogDbClient import SatnogDbClient
from glouton.shared.logger import logger


class TelemetryRepo:
    def __init__(self, cmd, repos):
        self.TELEMETRY_URL = 'telemetry/'
        self.__client = SatnogDbClient()
        self.__repos = repos
        self.__cmd = cmd
        self.__threads = []

    def extract(self):
        params = self.__create_request_params()
        limit = int(self.__cmd.page_to)
        page = int(self.__cmd.page_from)
        while True:
            r = self.__client.get_from_base(
                self.TELEMETRY_URL, params)
            if r.status_code != 200:
                break

            logger.Info('scanning page...' + params['page'])
            self.__read_page(r.json(), self.__cmd.page_from, self.__cmd.page_to)
            page += 1
            if page > limit:
                break

            params['page'] = str(page)

        print('\ndownloading started (Ctrl + C to stop)...\t~(  ^o^)~')
        self.__create_workers_and_wait()

    def __create_workers_and_wait(self):
        for repo in self.__repos:
            self.__threads.extend(repo.create_worker())
        while self.__is_one_thread_alive():
            for t in self.__threads:
                # let's control to main thread every seconds (in order to be able to capture Ctrl + C if needed)
                t.join(1)

    def __is_one_thread_alive(self):
        for thread in self.__threads:
            if thread.is_alive():
                return True

        return False

    def __read_page(self, telemetries, page_from, page_to):
        for telemetry in telemetries:
            for repo in self.__repos:
                repo.register_command(
                    telemetry, page_from, page_to)

    def __create_request_params(self):
        return {'satellite': self.__cmd.norad_id,
        #'start': self.__cmd.start_date.isoformat(),
        #'end': self.__cmd.end_date.isoformat(),
        'observer': self.__cmd.observer,
        'transmitter': self.__cmd.transmitter,
        'app_source': self.__cmd.app_source,
        'page': self.__cmd.page_from,
        'format': 'json'}



from glouton.commands.module.moduleCommand import ModuleCommand

class ObservationModuleCommand(ModuleCommand):
    def __init__(self, params):
        ModuleCommand.__init__(self, params)

    def process(self):
        self.params.module.runAfterDownload(self.params.file_name, self.params.full_path, self.params.observation)



from glouton.services.module.moduleService import ModuleService
from glouton.repositories.frame.frameRepo import FrameRepo
from glouton.repositories.telemetry.telemetryRepo import TelemetryRepo

class TelemetryService:
    def __init__(self, cmd):
        self.__cmd = cmd
        self.__module_service = ModuleService(self.__cmd)
        repos = self.filter_repositories()
        self.__telemetry_repo = TelemetryRepo(self.__cmd, repos)

    def filter_repositories(self):
        downloadable_data_repos = []
        downloadable_data_repos.append(FrameRepo(self.__cmd.working_dir, self.__module_service.loadFrameModules()))
        
        return downloadable_data_repos

    def extract(self):
        self.__telemetry_repo.extract()


import os
import sys
import json

def read():
    config = None
    default_config = {
                "DEFAULT": {
                    "NETWORK_API_URL": "https://network.satnogs.org/api/",
                    "DB_API_URL": "https://db.satnogs.org/api/",
                    "HTTPS_PROXY": "",
                    "HTTP_PROXY": ""
                },
                "MODULES": {
                    "PAYLOAD": [],
                    "WATERFALL": [],
                    "DEMODDATA": [],
                    "FRAME": [],
                    "FOR_ALL_OBSERVATION": []
                },
                "LOGFILE": "glouton.log"
            }
    try:
        if os.path.exists(os.path.dirname(sys.argv[0]) + "/../glouton/config.json"):
            with open(os.path.dirname(sys.argv[0]) + "/../glouton/config.json", 'r') as f:
                config = json.load(f)
        else:
            config = default_config

    except Exception as eee:
        print("error: ", eee)
        config = default_config
    return config



from glouton.services.module.moduleService import ModuleService
from glouton.repositories.demoddata.demoddataRepo import DemoddataRepo
from glouton.repositories.payload.payloadRepo import PayloadRepo
from glouton.repositories.waterfall.waterfallRepo import WaterfallRepo
from glouton.repositories.observation.observationsRepo import ObservationRepo

class ObservationsService:
    def __init__(self, cmd):
        self.__cmd = cmd
        self.__module_service = ModuleService(self.__cmd)
        repos = self.filter_repositories()
        self.__observations_repo = ObservationRepo(self.__cmd, repos)

    def filter_repositories(self):
        downloadable_data_repos = []
        all = not self.__cmd.payloads and not self.__cmd.waterfalls and not self.__cmd.demoddata
        if all or self.__cmd.payloads:
            downloadable_data_repos.append(PayloadRepo(self.__cmd.working_dir, self.__module_service.loadPayloadModules()))
        if all == True or self.__cmd.waterfalls:
            downloadable_data_repos.append(WaterfallRepo(self.__cmd.working_dir, self.__module_service.loadWaterfallModules()))
        if all == True or self.__cmd.demoddata:
            downloadable_data_repos.append(DemoddataRepo(self.__cmd.working_dir, self.__module_service.loadDemoddataModules()))
        
        return downloadable_data_repos

    def extract(self):
        self.__observations_repo.extract()




from glouton.modules.telemetryModuleBase import TelemetryModuleBase
from glouton.shared.logger import logger
import json
import os


class Polaris(TelemetryModuleBase):
    def __init__(self, wdir):
        TelemetryModuleBase.__init__(self, wdir)
        self.count = 0

    def runAfterDownload(self, frame, full_path, telemetry):
        try:
            timestamp = telemetry['timestamp']
            if not frame:
                print('no frame for ' + timestamp)

            json_file = full_path + '/' + timestamp.replace(':', '-') + '.json'
            if os.path.exists(json_file):
                with open(json_file) as json_file_read:
                    telemetry = json.load(json_file_read)
                    telemetry['telemetry'].append({
                        "timestamp": timestamp,
                        "frame": frame
                    })
                telemetry_obj = telemetry
            else:
                telemetry_obj = {"telemetry": [{
                    "timestamp": timestamp,
                    "frame": frame
                }]}

            json_telemetry = json.dumps(telemetry_obj)
            with open(json_file, 'w') as f:
                f.write(json_telemetry)

            self.count += 1
            print('Timestamp ' + timestamp + ' Frame ' +
                frame + ' count ' + str(self.count))
        except Exception as ex:
            logger.Error(ex)



from glouton.services.telemetry.telemetryService import TelemetryService
from glouton.repositories.frame.frameRepo import FrameRepo
from glouton.domain.parameters.programCmd import ProgramCmd

def test_loading_all_repository_types():
    params = ProgramCmd(None, None, None, None, None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None)

    service = TelemetryService(params)
    repos = service.filter_repositories()
    assert len(repos) == 1
    assert isinstance(repos[0], FrameRepo)


from glouton.services.observation.observationsService import ObservationsService
from glouton.repositories.payload.payloadRepo import PayloadRepo
from glouton.repositories.waterfall.waterfallRepo import WaterfallRepo
from glouton.repositories.demoddata.demoddataRepo import DemoddataRepo
from glouton.domain.parameters.programCmd import ProgramCmd

def test_loading_all_repository_types():
    params = ProgramCmd(None, None, None, None, None,
    None,
    True,
    True,
    True,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None)

    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 3
    assert isinstance(repos[0], PayloadRepo)
    assert isinstance(repos[1], WaterfallRepo)
    assert isinstance(repos[2], DemoddataRepo)

def test_loading_only_payload_repository_types():
    params = ProgramCmd(None, None, None, None, None,
    None,
    True,
    False,
    False,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None)
    
    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 1
    assert isinstance(repos[0], PayloadRepo)

def test_loading_only_waterfall_repository_types():
    params = ProgramCmd(None, None, None, None, None,
    None,
    False,
    True,
    False,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None)
        
    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 1
    assert isinstance(repos[0], WaterfallRepo)

def test_loading_only_Demoddata_repository_types():
    params = ProgramCmd(None, None, None, None, None,
    None,
    False,
    False,
    True,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None)
            
    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 1
    assert isinstance(repos[0], DemoddataRepo)


def test_loading_only_Payload_Demoddata_repository_types():
    params = ProgramCmd(None, None, None, None, None,
    None,
    True,
    False,
    True,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None)
                
    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 2
    assert isinstance(repos[0], PayloadRepo)
    assert isinstance(repos[1], DemoddataRepo)


def test_loading_only_Waterfall_Demoddata_repository_types():
    params = ProgramCmd(None, None, None, None, None,
    None,
    False,
    True,
    True,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None)
                    
    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 2
    assert isinstance(repos[0], WaterfallRepo)
    assert isinstance(repos[1], DemoddataRepo)

def test_default_loading_repository():
    params = ProgramCmd(None, None, None, None, None,
    None,
    False,
    False,
    False,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None,
    None)
                        
    service = ObservationsService(params)
    repos = service.filter_repositories()
    assert len(repos) == 3
    assert isinstance(repos[0], PayloadRepo)
    assert isinstance(repos[1], WaterfallRepo)
    assert isinstance(repos[2], DemoddataRepo)


class ProgramCmd:
    def __init__(self,
    norad_id,
    ground_station_id,
    start_date,
    end_date,
    observation_status,
    working_dir,
    payloads,
    waterfalls,
    demoddata,
    payload_modules,
    demoddata_modules,
    waterfall_modules,
    user,
    transmitter_uuid,
    transmitter_mode,
    transmitter_type,
    frame_modules,
    observer,
    app_source,
    transmitter,
    page_from,
    page_to):
        self.norad_id = norad_id
        self.ground_station_id = ground_station_id
        self.start_date = start_date
        self.end_date = end_date
        self.working_dir = working_dir
        self.payloads = payloads
        self.waterfalls = waterfalls
        self.demoddata = demoddata
        self.payload_modules = payload_modules
        self.demoddata_modules = demoddata_modules
        self.waterfall_modules = waterfall_modules
        self.observation_status = observation_status
        self.user = user
        self.transmitter_uuid = transmitter_uuid
        self.transmitter_mode = transmitter_mode
        self.transmitter_type = transmitter_type
        self.frame_modules = frame_modules
        self.observer = observer
        self.app_source = app_source
        self.transmitter = transmitter
        self.page_from = page_from
        self.page_to = page_to


from glouton.commands.download.downloadObservationCommand import DownloadObservationCommand
from glouton.shared import fileHelper
from glouton.shared.logger import logger
import os
import ntpath
import requests


class PayloadDownloadCommand(DownloadObservationCommand):
    def __init__(self, params, observation, modules_commands):
        DownloadObservationCommand.__init__(self, params, observation, modules_commands)
        self.__json_id = "payload"

    def download(self):
        url = self.observation[self.__json_id]
        if not url:
            logger.Info('no payload found for the observation ' + str(self.observation['id']) + ' of ' + self.observation['start'])
            return

        fileHelper.create_dir_if_not_exist(self.full_path)
        file_name = ntpath.basename(url)
        full_path_file = self.full_path + os.path.sep + file_name
        if os.path.exists(full_path_file):
            logger.Warning('pass ' + file_name + '... file already exist')
            return
            
        r = self.client.get(url)
        if r.status_code == 200:
            logger.Info('downloading...' + file_name)
            with open(full_path_file, "wb") as file:
                file.write(r.content)
            self.runModulesAfterDownload(file_name)



from glouton.commands.module.moduleCommandParams import ModuleCommandParams

class TelemetryModuleCommandParams(ModuleCommandParams):
    def __init__(self, file_name=None, full_path=None, telemetry=None, module=None):
        ModuleCommandParams.__init__(self, file_name, full_path, module)
        self.telemetry = telemetry



from queue import Queue
from glouton.infrastructure.satnogNetworkClient import SatnogNetworkClient
from glouton.shared.logger import logger


class ObservationRepo:
    def __init__(self, cmd, repos):
        self.OBSERVATION_URL = 'observations/'
        self.__client = SatnogNetworkClient()
        self.__repos = repos
        self.__cmd = cmd
        self.__threads = []

    def extract(self):
        params = self.__create_request_params()
        page = 1
        while True:
            r = self.__client.get_from_base(
                self.OBSERVATION_URL, params)
            if r.status_code != 200:
                break

            logger.Info('scanning page...' + params['page'])
            self.__read_page(r.json(), self.__cmd.start_date, self.__cmd.end_date)
            page += 1
            params['page'] = str(page)

        print('\ndownloading started (Ctrl + C to stop)...\t~(  ^o^)~')
        self.__create_workers_and_wait()

    def __create_workers_and_wait(self):
        for repo in self.__repos:
            self.__threads.extend(repo.create_worker())
        while self.__is_one_thread_alive():
            for t in self.__threads:
                # let's control to main thread every seconds (in order to be able to capture Ctrl + C if needed)
                t.join(1)

    def __is_one_thread_alive(self):
        for thread in self.__threads:
            if thread.is_alive():
                return True

        return False

    def __read_page(self, observations, start_date, end_date):
        for observation in observations:
            for repo in self.__repos:
                repo.register_command(
                    observation, start_date, end_date)

    def __create_request_params(self):
        return {'satellite__norad_cat_id': self.__cmd.norad_id,
        'ground_station': self.__cmd.ground_station_id,
        'start': self.__cmd.start_date.isoformat(),
        'end': self.__cmd.end_date.isoformat(),
        'vetted_status': self.__cmd.observation_status,
        'vetted_user': self.__cmd.user,
        'transmitter_uuid': self.__cmd.transmitter_uuid,
        'transmitter_mode': self.__cmd.transmitter_mode,
        'transmitter_type': self.__cmd.transmitter_type,
        'page': '1',
        'format': 'json'}



from glouton.shared import config
import os
import requests


class SatnogClient:
    def __init__(self):
        self.config  = config.read()
        self.proxies = self._set_proxy()

    def _set_proxy(self):
        proxies = None
        if "http_proxy" in os.environ and "https_proxy" in os.environ:
            proxies = {
                'http': os.environ['http_proxy'],
                'https': os.environ['https_proxy']
            }
            return proxies
        if self.config['DEFAULT']['HTTP_PROXY'] != "" or self.config['DEFAULT']['HTTPS_PROXY'] != "":
            proxies = {
                'http': self.config['DEFAULT']['HTTP_PROXY'],
                'https': self.config['DEFAULT']['HTTPS_PROXY']
            }

        return proxies

    def get(self, url, params=None):
        raise NotImplementedError()

    def get_from_base(self, url, params=None):
        raise NotImplementedError()



from glouton.infrastructure.satnogClient import SatnogClient
import os
import requests


class SatnogDbClient(SatnogClient):
    def __init__(self):
        SatnogClient.__init__(self)
        self._url = self.config['DEFAULT']['DB_API_URL']

    def get(self, url, params=None):
        return requests.get(url, params=params, proxies=self.proxies)

    def get_from_base(self, url, params=None):
        return requests.get(self._url + url, params=params, proxies=self.proxies)


from queue import Queue
from threading import Thread
from glouton.commands.download.downloadCommandParams import DownloadCommandParams
from glouton.commands.download.frameDownloadCommand import FrameDownloadCommand
from glouton.workers.downloadWorker import DownloadWorker
from glouton.workers.moduleWorker import ModuleWorker
from glouton.domain.interfaces.downloadable import Downloadable
from glouton.shared import threadHelper
from threading import Event

class FrameRepo(Downloadable):
    def __init__(self, working_dir, modules):
        self.__working_dir = working_dir
        self.__frame_commands = Queue()
        self.__frame_modules_commands = Queue()
        self.__modules = modules
        self.__download_status = Event()

    def register_command(self, telemetry, page_from, page_to):
        cmd_parameters = DownloadCommandParams(
            self.__working_dir, self.__create_dir_name('frames', page_from, page_to), self.__modules)
        waterfallDownloadCommand = FrameDownloadCommand(
            cmd_parameters, telemetry, self.__frame_modules_commands)
        self.__frame_commands.put(waterfallDownloadCommand)

    def create_worker(self):
        threads = []
        downloadWorker = DownloadWorker(self.__frame_commands, self.__download_status)
        threads.append(threadHelper.create_thread(downloadWorker.execute))
        if self.__modules is not None:
            moduleWorker = ModuleWorker(self.__frame_modules_commands, self.__download_status)
            threads.append(threadHelper.create_thread(moduleWorker.execute))

        return threads

    def __create_dir_name(self, target, page_from, page_to):
        return target + '__page__' + page_from + '__to__' + page_to



import importlib
from glouton.shared.logger import logger
from glouton.shared import config

class ModuleService:
    def __init__(self, cmd):
        self.__cmd = cmd
        self.__config = config.read()

    def loadDemoddataModules(self):
        logger.Info('Demoddata module(s) loading :')
        self.__cmd.demoddata_modules = self.__getModulesFromConfig(self.__cmd.demoddata_modules, 'DEMODDATA')
        self.__cmd.demoddata_modules = self.__getModulesFromConfig(self.__cmd.demoddata_modules, 'FOR_ALL_OBSERVATION')
        
        return self.__loadModule(self.__cmd.demoddata_modules)

    def loadPayloadModules(self):
        logger.Info('Payload module(s) loading :')
        self.__cmd.payload_modules = self.__getModulesFromConfig(self.__cmd.payload_modules, 'PAYLOAD')
        self.__cmd.payload_modules = self.__getModulesFromConfig(self.__cmd.payload_modules, 'FOR_ALL_OBSERVATION')

        return self.__loadModule(self.__cmd.payload_modules)

    def loadWaterfallModules(self):
        logger.Info('Waterfall module(s) loading :')
        self.__cmd.waterfall_modules = self.__getModulesFromConfig(self.__cmd.waterfall_modules, 'WATERFALL')
        self.__cmd.waterfall_modules = self.__getModulesFromConfig(self.__cmd.waterfall_modules, 'FOR_ALL_OBSERVATION')

        return self.__loadModule(self.__cmd.waterfall_modules)

    def loadFrameModules(self):
        logger.Info('Frame module(s) loading :')
        self.__cmd.frame_modules = self.__getModulesFromConfig(self.__cmd.frame_modules, 'FRAME')

        return self.__loadModule(self.__cmd.frame_modules)

    def __loadModule(self, modules):
        if modules is None:
            logger.Info('No module list found')
            return None
        
        loaded_modules = []
        for name in modules:
            loaded_module = importlib.import_module('glouton.modules.' + name.lower())
            module = getattr(loaded_module, name)
            loaded_modules.append(module(self.__cmd.working_dir))
            logger.Info('module : ' + name + ' loaded')
        return loaded_modules

    def __getModulesFromConfig(self, modules, config_array_name):
        try:
            modules_from_config = self.__config['MODULES'][config_array_name]
        except:
            logger.Warning('config.json : modules bad format')
            modules_from_config = []
        
        if len(modules_from_config) == 0 and modules is None:
            return None

        if len(modules_from_config) == 0 and modules is not None:
            return modules

        if modules is None:
            modules = []

        for module in modules_from_config:
            if module in modules:
                logger.Warning('warning : ' + module + ' already referenced.')

            modules.append(module)

        return modules



import mock
from glouton.shared.config import read

default_config = {
                "DEFAULT": {
                    "NETWORK_API_URL": "https://network.satnogs.org/api/",
                    "DB_API_URL": "https://db.satnogs.org/api/",
                    "HTTPS_PROXY": "",
                    "HTTP_PROXY": ""
                },
                "MODULES": {
                    "PAYLOAD": [],
                    "WATERFALL": [],
                    "DEMODDATA": [],
                    "FRAME": [],
                    "FOR_ALL_OBSERVATION": []
                },
                "LOGFILE": "glouton.log"
            }

def exist_return_false():
    return False

def test_default_configuration():
    patcher = mock.patch('os.path.exists')
    exists_mock = patcher.start()
    exists_mock.side_effect = exist_return_false
    config_obj = read()

    assert config_obj == default_config


from glouton.commands.download.downloadObservationCommand import DownloadObservationCommand
from glouton.shared import fileHelper
from glouton.shared.logger import logger
import os
import ntpath
import requests


class DemoddataDownloadCommand(DownloadObservationCommand):
    def __init__(self, params, observation, modules_commands):
        DownloadObservationCommand.__init__(self, params, observation, modules_commands)
        self.__json_id = "demoddata"

    def download(self):
        demoddata = self.observation[self.__json_id]
        if not any(demoddata):
            logger.Info('no demoddata found for the observation ' + str(self.observation['id']) + ' of ' + self.observation['start'])
            return

        fileHelper.create_dir_if_not_exist(self.full_path)
        for demod in demoddata:
            file_name = ntpath.basename(demod['payload_demod'])
            full_path_file = self.full_path + os.path.sep + file_name
            if os.path.exists(full_path_file):
                logger.Warning('pass ' + file_name + '... file already exist')
                return

            r = self.client.get(demod['payload_demod'])
            if r.status_code == 200:
                logger.Info('downloading...' + file_name)
                with open(full_path_file, "wb") as file:
                    file.write(r.content)
                self.runModulesAfterDownload(file_name)



""" Glouton setup.

"""

from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    version="0.4.1",
    name="glouton",
    description="`Glouton is a cli program which helps you downloading satnogs data`",
    url="https://github.com/deckbsd/glouton-satnogs-data-downloader.git",
    license="MIT",
    author="deckbsd",
    install_requires=["requests"],
    python_requires='>=3',
    extras_require={"test": ["pytest"]},
    packages=find_packages(exclude=["tests", "docs"]),
    data_files=[('glouton', ['glouton/config.json'])],
    scripts=['bin/glouton'],
    keywords="satnogs, telemetry, satellite, payload, data, db, download",
    classifiers=[
        "Development Status :: 3 - Alpha", "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License"
    ],
)



from glouton.modules.moduleBase import ModuleBase

class TelemetryModuleBase(ModuleBase):
    def __init__(self, working_dir):
        ModuleBase.__init__(self, working_dir)

    def runAfterDownload(self, frame, full_path, telemetry):
        raise NotImplementedError()


class ModuleWorker:
    def __init__(self, queue, download_status):
        self.__commands = queue
        self.__download_status = download_status

    def execute(self):
        while self.__commands.empty() == False or self.__download_status.isSet():
            try:
                command = self.__commands.get(block=True, timeout= 1)
                command.process()
                self.__commands.task_done()
            except:
                pass


import logging
from glouton.shared import config


class Logger:
    def __init__(self) -> None:
        if config is not None:
            self.__config = _config = config.read()
        if self.__config is not None:
            logging.basicConfig(filename=self.__config['LOGFILE'],
                                format='%(asctime)s %(levelname)s:%(message)s',
                                datefmt='%m/%d/%Y %I:%M:%S %p',
                                level=logging.DEBUG)

    def Info(self, info) -> None:
        print(info)
        logging.info(info)

    def Error(self, error) -> None:
        print(error)
        logging.error(error)

    def Debug(self, str) -> None:
        print(str)
        logging.debug(str)

    def Warning(self, warning) -> None:
        print(warning)
        logging.warning(warning)

logger = Logger()
